#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <limits.h>

#if KYSEC
// #include <kysec/libkysec_dbus.h>
#include <kysec/status_dbus.h>
#endif
#include <errno.h>

#include "whitelst_db.h"
#include "wlist.h"

extern char *get_package_name(char *debname);

static int _parse_file_type(char *file_type)
{
	int type = -1;

	if (strcmp(file_type, EXE_TYPE) == 0)
		type = 0;
	else if (strcmp(file_type, SO_TYPE) == 0)
		type = 2;
	else if (strcmp(file_type, KO_TYPE) == 0)
		type = 3;
	else if (strcmp(file_type, SCRIPT_TYPE) == 0)
		type = 1;
	else if (strcmp(file_type, APP_TYPE) == 0)
		type = 5;

	return type;
}

int wlist_insert_policy(char *file_path, char *file_type)
{
	int type = -1;
	int ret = -1;

	type = _parse_file_type(file_type);
	if (type == -1)
	{
		log_error("The type[%s] is an invalid!\n", file_type);
		return -1;
	}

	char absolute_path[PATH_MAX];
    realpath(file_path, absolute_path);

	if (access(absolute_path, F_OK) < 0)
	{
		log_error("The file[%s] does not exist!\n", file_path);
		return -2;
	}

	ret = whitelstAdd(type, absolute_path, 0, 0);

	return ret;
}

int wlist_delete_policy(char *file_path, char *file_type)
{
	int type = -1;
	int ret = -1;

	type = _parse_file_type(file_type);
	if (type == -1)
	{
		log_error("The type[%s] is an invalid!\n", file_type);
		return -1;
	}

	char absolute_path[PATH_MAX];
    realpath(file_path, absolute_path);

#if 0
	if(access(file_path, F_OK) < 0){
		log_error("The file[%s] does not exist!\n", file_path);
		return -2;
	}
#endif
	/* measure */

	ret = whitelstDelete(type, absolute_path);

	return ret;
}

int wlist_add_app_policy(char *appname)
{
	return whitelstAddByDeb(appname);
}

int wlist_del_app_policy(char *appname)
{
	return whitelstDeleteByDeb(appname);
}

int wlist_add_file_protect(char *file_name)
{
	int ret = -1;

	char absolute_path[PATH_MAX];
    realpath(file_name, absolute_path);

	struct stat sb;
	if (stat(absolute_path, &sb) < 0)
	{
		log_error("The file[%s]: %s\n", file_name, strerror(errno));
		return -1;
	}

	switch (sb.st_mode & S_IFMT)
	{
	case S_IFDIR:
		ret = whitelstAddByDir(4, absolute_path);
		break;
	case S_IFREG:
		ret = whitelstAdd(4, absolute_path, 0, 0);
		break;
	default:
	{
		log_error("Unknown file[%s] type\n", file_name);
		return -1;
	}
	}

	return ret;
}

int wlist_remove_file_protect(char *file_name)
{
	int ret = -1;

	char absolute_path[PATH_MAX];
    realpath(file_name, absolute_path);

	struct stat sb;
	if (stat(absolute_path, &sb) < 0)
	{
		log_error("The file[%s]: %s\n", file_name, strerror(errno));
		return -1;
	}

	switch (sb.st_mode & S_IFMT)
	{
	case S_IFDIR:
		ret = whitelstDeleteByDir(4, absolute_path);
		break;
	case S_IFREG:
		ret = whitelstDelete(4, absolute_path);
		break;
	default:
	{
		log_error("Unknown file[%s] type\n", file_name);
		return -1;
	}
	}

	return ret;
}

int set_wlist_status(int status)
{
#if KYSEC
	if (status == 0)
	{
		if (kysec_setstatus(4) != 0)
		{
			audit_record_for_kysec("status_off", 0);
			return -1;
		}
		else
			audit_record_for_kysec("status_off", 1);
	}
	else if (status == 1)
	{
		if (kysec_setstatus(2) != 0)
		{
			audit_record_for_kysec("status_on", 0);
			return -1;
		}
		else
			audit_record_for_kysec("status_on", 1);
	}
	else
	{
		errno = EINVAL;
		return -1;
	}
#endif
	return 0;
}

static void write_whitelist_file(int ftype, FILE *fp)
{
	int count = 0, i = 0;
	struct white_list *wlst = NULL;
	char sm3_data[PATH_MAX + SHA1_SIZE + 10] = {0};

	// printf("sm3_data length = %d\n", sizeof(sm3_data));

	if (fp == NULL || ftype == -1)
	{
		log_error("Invalid arguments in %s\n", __func__);
		return;
	}

	count = getWhitelstCount(ftype, NULL);
	if (count <= 0)
		return;

	wlst = whitelstShowByType(ftype, count, NULL);
	if (wlst == NULL)
		return;

	for (i = 0; i < count; i++)
	{
		memset(sm3_data, 0, sizeof(sm3_data));
		snprintf(sm3_data, sizeof(sm3_data), "%s\t%s\n",
				 wlst[i].hash, wlst[i].fpath);

		fwrite(sm3_data, sizeof(char), strlen(sm3_data), fp);
	}

	if (!wlst)
	{
		free(wlst);
		wlst = NULL;
	}
}

/* get whitelist from db, and write the data to WHLIST_SM3 */
int create_sm3_whitelist_file(void)
{
	FILE *fp = NULL;
	int ftype = -1;

	fp = fopen(WHLIST_SM3, "w+");
	if (fp == NULL)
	{
		log_error("Can't open %s: %s\n", WHLIST_SM3, strerror(errno));
		return -1;
	}

	/* TYPE_BINARY is 0, and TYPE_KMOD is 3 */
	for (ftype = TYPE_BINARY; ftype <= TYPE_KMOD; ftype++)
		write_whitelist_file(ftype, fp);

	/* get other type(6) */
	// write_whitelist_file(6, fp);

	fclose(fp);
	return 0;
}
